from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _


class Watermark(models.Model):
    OPACITY_ERROR_TEXT = _(u'Opacity should be more 0 and less 1. This is a rule!')

    TILE = 'tile'
    SCALE = 'scale'
    BR = 'br'
    TR = 'tr'
    BL = 'bl'
    TL = 'tl'

    POSITIONS = (
        (TILE, _(u'tile')),
        (SCALE, _(u'scale')),
        (BR, _(u'buttom right corner')),
        (TR, _(u'top rigth corner')),
        (BL, _(u'buttom left corner')),
        (TL, _(u'top left corner')),
    )

    title = models.CharField(max_length=32, verbose_name=_(u'Title'), unique=True)
    mark = models.ImageField(upload_to='watermarks', verbose_name=_(u'Watermark'))
    opacity = models.FloatField(default=1, verbose_name=_(u'Opacity'), help_text=_(u'Value must be between 0 and 1'))
    position = models.CharField(max_length=8, verbose_name=_(u'Position'), choices=POSITIONS)
    x = models.IntegerField(blank=True, null=True, verbose_name=_(u'Indent X'), default=0)
    y = models.IntegerField(blank=True, null=True, verbose_name=_(u'Indent Y'), default=0)
    is_active = models.BooleanField(default=True, verbose_name=_(u'Is active'))
    update_hard = models.BooleanField(default=False, verbose_name=_(u'Update hard'),
                                      help_text=_(u'Use it if you want to update all already created watermarks'))

    def get_position(self):
        return self.get_position_display()

    def clean(self):
        if not 0 <= self.opacity <= 1:
            raise ValidationError(self.OPACITY_ERROR_TEXT)

    def __unicode__(self):
        return self.title

    get_position.short_description = _(u'Position')

    class Meta:
        verbose_name = _(u'Watermark')
        verbose_name_plural = _(u'Watermarks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Watermark',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(unique=True, max_length=32, verbose_name='Title')),
                ('mark', models.ImageField(upload_to=b'watermarks', verbose_name='Watermark')),
                ('opacity', models.FloatField(default=1, help_text='Value must be between 0 and 1', verbose_name='Opacity')),
                ('position', models.CharField(max_length=8, verbose_name='Position', choices=[(b'tile', 'tile'), (b'scale', 'scale'), (b'br', 'buttom right corner'), (b'tr', 'top rigth corner'), (b'bl', 'buttom left corner'), (b'tl', 'top left corner')])),
                ('x', models.IntegerField(default=0, null=True, verbose_name='Indent X', blank=True)),
                ('y', models.IntegerField(default=0, null=True, verbose_name='Indent Y', blank=True)),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('update_hard', models.BooleanField(default=False, help_text='Use it if you want to update all already created watermarks', verbose_name='Update hard')),
            ],
            options={
                'verbose_name': 'Watermark',
                'verbose_name_plural': 'Watermarks',
            },
            bases=(models.Model,),
        ),
    ]



from django.contrib import admin

from watermarker import models


class WatermarkAdmin(admin.ModelAdmin):
    list_display = ('title', 'get_position', 'opacity', 'is_active')
    search_fields = ('title',)
    fieldsets = (
        (None, {'fields': ('title', 'mark', 'opacity', 'position', ('x', 'y'), 'is_active', 'update_hard'),
                'classes': ('wide',)}),
    )


admin.site.register(models.Watermark, WatermarkAdmin)



# coding: utf-8
import os
import urllib
from logging import error

from django.db.models.fields.files import ImageFieldFile, ImageField
from django.conf import settings
from PIL import Image, ImageEnhance

from watermarker import models


def reduce_opacity(im, opacity):
    """Returns an image with reduced opacity."""

    assert opacity >= 0 and opacity <= 1
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    else:
        im = im.copy()
    alpha = im.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    im.putalpha(alpha)
    return im


def make_watermark(im, mark, position, opacity=1, shift=(0, 0)):
    """Adds a watermark to an image."""

    if opacity < 1:
        mark = reduce_opacity(mark, opacity)
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    # create a transparent layer the size of the image and draw the
    # watermark in that layer.
    layer = Image.new('RGBA', im.size, (0, 0, 0, 0))
    if position == 'tile':
        for y in range(0, im.size[1], mark.size[1]):
            for x in range(0, im.size[0], mark.size[0]):
                layer.paste(mark, (x, y))
    elif position == 'scale':
        # scale, but preserve the aspect ratio
        ratio = min(
            float(im.size[0]) / mark.size[0], float(im.size[1]) / mark.size[1])
        w = int(mark.size[0] * ratio)
        h = int(mark.size[1] * ratio)
        mark = mark.resize((w, h))
        layer.paste(mark, ((im.size[0] - w) / 2, (im.size[1] - h) / 2))
    elif position == 'br':
        x = max(im.size[0] - mark.size[0], 0) - shift[0]
        y = max(im.size[1] - mark.size[1], 0) - shift[1]
        layer.paste(mark, (x, y))
    elif position == 'tr':
        x = max(im.size[0] - mark.size[0], 0) - shift[0]
        y = 0 + shift[1]
        layer.paste(mark, (x, y))
    elif position == 'bl':
        x = 0 + shift[0]
        y = max(im.size[1] - mark.size[1], 0) - shift[1]
        layer.paste(mark, (x, y))
    elif position == 'tl':
        x = 0 + shift[0]
        y = 0 + shift[1]
        layer.paste(mark, (x, y))
    else:
        layer.paste(mark, (0, 0))
    # composite the watermark with the layer
    return Image.composite(layer, im, layer)


def get_path(url):
    """Makes a filesystem path from the specified URL"""

    if url.startswith(settings.MEDIA_URL):
        root = settings.MEDIA_ROOT
    else:
        root = settings.STATIC_ROOT

    url = url.replace(settings.MEDIA_URL, '').replace(settings.STATIC_URL, '')
    url = urllib.unquote(url).decode("utf8")
    return os.path.normpath(os.path.join(root, url))


def make_iff_instance(path):
    """ Returns ImageFieldFile for provided image path """

    return ImageFieldFile(instance=None, field=ImageField(), name=path)


def watermark(url, wm):
    # return usual url(string) or ImageFieldFile
    result_as_iff = False

    if isinstance(wm, models.Watermark):
        watermark = wm
    else:
        try:
            watermark = models.Watermark.objects.get(title=wm, is_active=True)
        except (models.Watermark.DoesNotExist, models.Watermark.MultipleObjectsReturned):
            return url

    if not watermark.is_active:
        return url

    # to work not only with strings
    if isinstance(url, ImageFieldFile):
        result_as_iff = True
        if hasattr(url, 'url'):
            url = url.url
        else:
            return url

    basedir = '%s/watermarked' % os.path.dirname(url)
    base, ext = os.path.splitext(os.path.basename(url))

    # watermarked url for template
    wm_url = os.path.join(basedir, '%s%s' % (base, ext))
    # path to save watermarked img
    wm_path = get_path(wm_url)

    # not to do more than we need
    if os.path.exists(wm_path) and not watermark.update_hard:
        if result_as_iff:
            return make_iff_instance(wm_path)
        return wm_url

    # create a folder for watermarks if needed
    wm_dir = os.path.dirname(wm_path)
    if not os.path.exists(wm_dir):
        os.makedirs(wm_dir)

    img_path = get_path(url)

    if not os.path.exists(img_path):
        return url

    try:
        img = Image.open(img_path)

        mark = Image.open(watermark.mark.path)
        x = watermark.x or 0
        y = watermark.y or 0
        shift = (x, y)
        position = watermark.position

        opacity = watermark.opacity

        marked_img = make_watermark(img, mark, position, opacity, shift)
        marked_img.save(wm_path)
    except Exception, e:
        error("Cant create watermark for %s. Error type: %s. Msg: %s" % (img_path, type(e), e))
        return url

    if result_as_iff:
        return make_iff_instance(wm_path)

    return wm_url


def get_url_safe(path):
    """ If we save wm with result_as_iff == True, we thumbnail returns us full path to picture (not valid url)
    may be there is prettier way to manage this

    """

    if not path.startswith(settings.MEDIA_URL):
        return os.path.join(settings.MEDIA_URL, path.split(settings.MEDIA_URL)[-1])
    return path



import os

from distutils.core import setup

import watermarker

with open('pypi_doc.txt') as file:
    long_description = file.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-watermarker',
    version=watermarker.__version__,
    packages=[
        'watermarker',
        'watermarker.templatetags',
        'watermarker.locale',
        'watermarker.locale.ru',
        'watermarker.locale.ru.LC_MESSAGES',
    ],
    url='https://github.com/Skycker/watermarker',
    license='BSD License',
    author='Kirill Kostyukhin',
    description='A tool for easy working with watermarks in django projects',
    long_description=long_description,
    keywords="django, watermark, watermarks, watermarker, image",
    install_requires=['pillow'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Internet :: WWW/HTTP",
    ],
)
